<?php

namespace Drupal\tg_integration;

use Drupal\Core\Link;

class TgIntegrationCommands {

  const BASE_TG_URL = 'https://api.telegram.org/bot';

  /**
   * Send text messages to the Telegram channel.
   *
   * @param array $parameters
   *   The array of parameters. 'chat_id' not needed.
   * @see https://core.telegram.org/bots/api#sendmessage.
   *
   * @return int|null
   *   The message_id from 'Message' object
   * @see https://core.telegram.org/bots/api#message.
   */
  public function sendMessage(array $parameters): ?int {
    $response = $this->doRequest('sendMessage', $parameters);
    return $response['result']['message_id'] ?? NULL;

  }

  /**
   * Get the number of members in the Telegram chat. Returns Int on success.
   *
   * @return int
   * @see https://core.telegram.org/bots/api#getchatmembercount.
   */
  public function getChatMemberCount(): int {
    $response = $this->doRequest('getChatMemberCount');
    return $response['result'];
  }

  /**
   * Send request to the Telegram bot API.
   *  @see https://core.telegram.org/bots/api.
   *
   * @param string $command
   *   The string with API method name.
   * @param array $parameters
   *   The array of parameters fot the method. 'chat_id' not needed.
   *
   * @return array
   *   The array with response.
   */
  public function doRequest(string $command, array $parameters = []): array {
    // 	@TODO Exception: Drupal\Core\Entity\EntityStorageException: cURL error 6: Could not resolve host: api.telegram.org (see https://curl.haxx.se/libcurl/c/libcurl-errors.html) in Drupal\Core\Entity\Sql\SqlContentEntityStorage->save() (line 811 of /var/www/html/web/core/lib/Drupal/Core/Entity/Sql/SqlContentEntityStorage.php).
    $config = \Drupal::config('tg_integration.settings');
    $token = $config->get('token');
    $chat_name = $config->get('chat_name');
    if (!$token || !$chat_name) {
      $link = Link::createFromRoute('settings', 'tg_integration.settings')->toString();
      \Drupal::messenger()->addWarning(t('Request to Telegram was not done. Please fill @settings with correct data.', ['@settings' => $link]));
      return [];
    }

    $request_url = self::BASE_TG_URL . $token . '/' . $command;
    $options = ['query' => $parameters];
    $options['query']['chat_id' ] = '@' . $chat_name;
    $client = \Drupal::httpClient();
    $response = $client->request('GET', $request_url, $options);
    if ($response->getStatusCode() == 200) {
      return json_decode($response->getBody()->getContents(), TRUE);
    }
    return [];
  }

}
