<?php

namespace Drupal\tg_integration;

/**
 * Provides an interface defining tg_integration Storage.
 *
 * Stores data for the post a node to the Telegram.
 */
interface TgIntegrationStorageInterface {

  /**
   * Write data about the post to the database.
   *
   * @param string $entity_type
   *   The type of the entity which posted to the Telegram.
   * @param int $entity_id
   *   The ID of the entity which posted to the Telegram.
   * @param int $tg_post_id
   *   The message_id in the response from the Telegram.
   * @see https://core.telegram.org/bots/api#message.
   * @param bool $display_comments
   *   TRUE (default) to display Telegram comments block under the node;
   *   FALSE to disable Telegram comments block under the node.
   *
   * @return bool
   *   TRUE if the data about the post has been written to the database.
   */
  public function setTgPost(
    string $entity_type,
    int $entity_id,
    int $tg_post_id,
    bool $display_comments = true
  ): bool;

  /**
   * Get Post id in the Telegram (message_id from request) for entity.
   * @see https://core.telegram.org/bots/api#message.
   *
   * @param string $entity_type
   *   The type of the entity which posted to the Telegram.
   * @param int $entity_id
   *   The ID of the entity which posted to the Telegram.
   *
   * @return int|NULL
   *   The post id in the Telegram (message_id from request) for current entity.
   * @see https://core.telegram.org/bots/api#message.
   */
  public function getTgPostId(string $entity_type, int $entity_id): ?int;

  /**
   * Check if the current entity has been published to the Telegram chanel.
   *
   * @param string $entity_type
   *   The type of the entity being checked.
   * @param int $entity_id
   *   The ID of the entity being checked.
   *
   * @return bool
   *   TRUE if entity has been posted to the Telegram chanel;
   *   FALSE if entity hasn't been posted to the Telegram chanel;
   */
  public function hasTgPost(string $entity_type, int $entity_id): bool;

  /**
   * Get path for the post in the Telegram for the entity in format
   * 'chanel_name/post_id'. F.e. 'finews_ua/56'.
   *
   * @param string $entity_type
   *   The type of the entity which posted to the Telegram.
   * @param int $entity_id
   *   The ID of the entity which posted to the Telegram.
   *
   * @return string|NULL
   *   The path for the post in the Telegram for the entity in format
   * 'chanel_name/post_id'. NULL if the current entity hasn't been posted to
   * the Telegram chanel.
   */
  public function getTgPostPath(string $entity_type, int $entity_id): ?string;

  /**
   * Get status display Telegram comments block under the node.
   *
   * @param string $entity_type
   *   The type of the entity which posted to the Telegram.
   * @param int $entity_id
   *   The ID of the entity which posted to the Telegram.
   *
   * @return bool|NULL
   *   TRUE to display Telegram comments block under the node;
   *   FALSE to disable Telegram comments block under the node;
   *   NULL if the current entity hasn't been posted to the Telegram chanel.
   */
  public function getCommentsStatus(string $entity_type, int $entity_id): ?bool;

  /**
   * Change whether display Telegram comments block under the node or not.
   *
   * @param string $entity_type
   *   The type of the entity which posted to the Telegram.
   * @param int $entity_id
   *   The ID of the entity which posted to the Telegram.
   * @param bool $display_comments
   *   TRUE to display Telegram comments block under the node;
   *   FALSE to disable Telegram comments block under the node.
   *
   * @return bool
   *   TRUE if the data about comments displaying has been changed.
   */
  public function changeCommentsStatus(
    string $entity_type,
    int $entity_id,
    bool $display_comments
  ): bool;

}
